==> mars-0.9.2.tar.gz: mars-0.9.2/src/host/lib/mars_mutex.c <==
/*
 * Copyright 2008 Sony Corporation of America
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this Library and associated documentation files (the
 * "Library"), to deal in the Library without restriction, including
 * without limitation the rights to use, copy, modify, merge, publish,
 * distribute, sublicense, and/or sell copies of the Library, and to
 * permit persons to whom the Library is furnished to do so, subject to
 * the following conditions:
 *
 *  The above copyright notice and this permission notice shall be
 *  included in all copies or substantial portions of the Library.
 *
 *  If you modify the Library, you may copy and distribute your modified
 *  version of the Library in object code or as an executable provided
 *  that you also do one of the following:
 *
 *   Accompany the modified version of the Library with the complete
 *   corresponding machine-readable source code for the modified version
 *   of the Library; or,
 *
 *   Accompany the modified version of the Library with a written offer
 *   for a complete machine-readable copy of the corresponding source
 *   code of the modified version of the Library.
 *
 *
 * THE LIBRARY IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 * IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
 * CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
 * TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
 * LIBRARY OR THE USE OR OTHER DEALINGS IN THE LIBRARY.
 */

#include <ppu_intrinsics.h>

#include "mars/mars_mutex.h"
#include "mars/mars_error.h"
#include "mars/mars_debug.h"

int mars_mutex_initialize(struct mars_mutex *mutex)
{
	MARS_CHECK_RET(mutex, MARS_ERROR_NULL);
	MARS_CHECK_RET(((uintptr_t)mutex & MARS_MUTEX_ALIGN_MASK) == 0,
			MARS_ERROR_ALIGN);

	mutex->lock = MARS_MUTEX_UNLOCKED;
	__lwsync();

	return MARS_SUCCESS;
}

int mars_mutex_lock(struct mars_mutex *mutex)
{
	MARS_CHECK_RET(mutex, MARS_ERROR_NULL);
	MARS_CHECK_RET(((uintptr_t)mutex & MARS_MUTEX_ALIGN_MASK) == 0,
			MARS_ERROR_ALIGN);

	do {
		do {
		} while (__lwarx(&mutex->lock) == MARS_MUTEX_LOCKED);
	} while (!__stwcx(&mutex->lock, MARS_MUTEX_LOCKED));

	__isync();

	return MARS_SUCCESS;
}

int mars_mutex_unlock(struct mars_mutex *mutex)
{
	MARS_CHECK_RET(mutex, MARS_ERROR_NULL);
	MARS_CHECK_RET(((uintptr_t)mutex & MARS_MUTEX_ALIGN_MASK) == 0,
			MARS_ERROR_ALIGN);
	MARS_CHECK_RET(mutex->lock == MARS_MUTEX_LOCKED, MARS_ERROR_STATE);

	__lwsync();
	mutex->lock = MARS_MUTEX_UNLOCKED;

	return MARS_SUCCESS;
}

==> mars-08.06.26.tar.gz: mars-08.06.26/src/host/lib/mars_mutex.c <==
/*
 * Copyright 2008 Sony Corporation of America
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this Library and associated documentation files (the
 * "Library"), to deal in the Library without restriction, including
 * without limitation the rights to use, copy, modify, merge, publish,
 * distribute, sublicense, and/or sell copies of the Library, and to
 * permit persons to whom the Library is furnished to do so, subject to
 * the following conditions:
 *
 *  The above copyright notice and this permission notice shall be 
 *  included in all copies or substantial portions of the Library.
 *
 *  If you modify the Library, you may copy and distribute your modified 
 *  version of the Library in object code or as an executable provided 
 *  that you also do one of the following:
 *
 *   Accompany the modified version of the Library with the complete 
 *   corresponding machine-readable source code for the modified version 
 *   of the Library; or,
 *
 *   Accompany the modified version of the Library with a written offer 
 *   for a complete machine-readable copy of the corresponding source 
 *   code of the modified version of the Library.
 *
 *
 * THE LIBRARY IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 * IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
 * CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
 * TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
 * LIBRARY OR THE USE OR OTHER DEALINGS IN THE LIBRARY.
 */

#include "mars/mars_mutex.h"
#include "mars/mars_atomic.h"
#include "mars/mars_error.h"
#include "mars/mars_debug.h"

int mars_mutex_initialize(struct mars_mutex *mutex)
{
	MARS_CHECK_RET(mutex, MARS_ERROR_NULL);
	MARS_CHECK_RET(((uintptr_t)mutex & MARS_MUTEX_ALIGN_MASK) == 0,
			MARS_ERROR_ALIGN);

	mutex->lock = MARS_MUTEX_UNLOCKED;

	return MARS_SUCCESS;
}

int mars_mutex_lock(struct mars_mutex *mutex)
{
	MARS_CHECK_RET(mutex, MARS_ERROR_NULL);
	MARS_CHECK_RET(((uintptr_t)mutex & MARS_MUTEX_ALIGN_MASK) == 0,
			MARS_ERROR_ALIGN);

	do {
		do {
		} while (atomic_get(&mutex->lock) == MARS_MUTEX_LOCKED);
	} while (!atomic_put(&mutex->lock, MARS_MUTEX_LOCKED));

	atomic_sync();

	return MARS_SUCCESS;
}

int mars_mutex_unlock(struct mars_mutex *mutex)
{
	MARS_CHECK_RET(mutex, MARS_ERROR_NULL);
	MARS_CHECK_RET(((uintptr_t)mutex & MARS_MUTEX_ALIGN_MASK) == 0,
			MARS_ERROR_ALIGN);
	MARS_CHECK_RET(mutex->lock == MARS_MUTEX_LOCKED, MARS_ERROR_STATE);

	mutex->lock = MARS_MUTEX_UNLOCKED;

	return MARS_SUCCESS;
}
